from django.core.paginator import Paginator, InvalidPage, EmptyPage
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.http import HttpResponseRedirect
from django.core.urlresolvers import reverse
from browse.forms import SearchFiltersForm
from releases.models import Distro
import urllib

default_search_form_data = {
    'release': '%s' % Distro.objects.all().order_by('-i_version')[0].id,
    'stars': '3',
    'desktops': 'on',
    'laptops': 'on',
    'popularity': 'any',
}

mobile_default_search_form_data = default_search_form_data.copy()
mobile_default_search_form_data['stars'] = '1'

PER_PAGE = 20
MOBILE_PER_PAGE = 4
ORDER_COLUMNS = ('make', 'model', 'overall_rating', 'num_ratings')


def _get_base_query_string(request, data):
    base_query_string = 'desktops=%s' + \
                        '&laptops=%s' + \
                        '&stars=%s' + \
                        '&release=%s' + \
                        '&popularity=%s' + \
                        '&term=%s'
    base_query_string = base_query_string % (
                            data['desktops'] or '',
                            data['laptops'] or '',
                            data['stars'],
                            data['release'].id,
                            data['popularity'],
                            urllib.quote_plus(request.GET.get('term', '')))
    base_query_string = base_query_string.replace('desktops=True',
                                                    'desktops=on') \
                                         .replace('laptops=True',
                                                    'laptops=on')
    return base_query_string


def _get_index_data(request):
    if request.mobile:
        search_data = mobile_default_search_form_data.copy()
    else:
        search_data = default_search_form_data.copy()
    if len(request.GET) > 0:
        incoming = {}
        if 'desktops' in search_data:
            del(search_data['desktops'])
        if 'laptops' in search_data:
            del(search_data['laptops'])
        for p in ('release', 'stars', 'desktops', 'laptops', 'popularity'):
            v = request.GET.get(p, False)
            if v:
                incoming[p] = v
        search_data.update(incoming)
    filter_form = SearchFiltersForm(search_data)
    results = filter_form.search(request.GET.get('term', None))
    order_column = request.GET.get('order', '')
    if order_column not in ORDER_COLUMNS:
        order_column = 'overall_rating'

    order_direction = request.GET.get('order_direction', 'desc')
    if order_direction != 'asc':
        order = '-' + order_column
    else:
        order = order_column

    results = results.order_by(order, 'make', 'model')
    per_page = PER_PAGE
    if request.mobile:
        per_page = MOBILE_PER_PAGE
    paginator = Paginator(results, per_page)

    try:
        page = int(request.GET.get('page', '1'))
    except ValueError:
        page = 1

    try:
        results = paginator.page(page)
    except (EmptyPage, InvalidPage):
        results = paginator.page(paginator.num_pages)

    form_data = filter_form.cleaned_data
    return {
        'results': results,
        'base_query_string': _get_base_query_string(request, form_data),
        'filter_form': filter_form,
        'release': form_data['release'],
        'order': order_column,
        'order_direction': order_direction,
    }


def index_mobile(request):
    template_data = _get_index_data(request)
    form_data = template_data['filter_form'].cleaned_data
    template_data['tabs'] = []
    releases = Distro.objects.all().order_by('-i_version')
    for idx, d in enumerate(releases):
        label = d.release
        if idx == 0:
            label = 'Latest'
        tab_url = reverse(index_mobile)
        query_data = form_data
        query_data['release'] = d
        tab_url = tab_url + '?' + _get_base_query_string(request, query_data)
        tab = {'label': label,
            'url': tab_url,
            'active': False}
        if request.GET.get('release', False) == str(d.id):
            tab['active'] = True
        if not request.GET.get('release', False) and idx == 0:
            tab['active'] = True
        template_data['tabs'].append(tab)
    context = RequestContext(request, template_data)
    if request.GET.get('term', False):
        return render_to_response('mobile/results.html', context)
    return render_to_response('mobile/index.html', context)


def index(request):
    if request.mobile:
        return HttpResponseRedirect(reverse(index_mobile))
    template_data = _get_index_data(request)
    context = RequestContext(request, template_data)
    return render_to_response('index.html', context)
