from django.conf import settings
from django.db.models import Q
from releases.models import Distro
from results.util import save_config_star_rating
from system_configs.models import (
    SystemConfig,
    SystemUnit,
    SystemDevice,
    SystemUnitTestResult,
    RejectedSubmission,
    SkippedTest,
    generate_device_signature,
    PASSED,
    FAILED,
    SKIPPED,
    UNKNOWN,
)
from results.models import (
    get_category_mappings,
    get_skippable_tests,
    get_test_mappings,
    get_lock,
)
import re
import sys

# categories that define a variation of a model
PROFILING_CATEGORIES = (
    'video',
    'cpu',
    'wireless',
)

CORE_DESKTOP = (
    'cpu',
    'video',
    'hard disk',
    'memory',
    'input devices',
    'audio',
    'usb controllers',
    'vga external video',
    'ethernet card',
    'optical devices',
)

CORE_LAPTOP = (
    'cpu',
    'video',
    'hard disk',
    'memory',
    'input devices',
    'audio',
    'usb controllers',
    'vga external video',
    'ethernet card',
    'suspend/resume',
    'wireless',
)

ADDITIONAL_DESKTOP = (
    'wireless',
    'suspend/resume',
    'hibernate/resume',
    'hdmi external video',
    'bluetooth',
    'mediacards',
    'firewire',
)

ADDITIONAL_LAPTOP = (
    'optical devices',
    'hibernate/resume',
    'hdmi external video',
    'bluetooth',
    'mediacards',
    'firewire',
    'camera',
    'special keys',
    'leds',
    'pcmcia',
)


def get_core_and_additional_categories(system):
    if system.form_factor == 'desktop':
        CORE = CORE_DESKTOP
        ADDITIONAL = ADDITIONAL_DESKTOP

    if system.form_factor == 'laptop':
        CORE = CORE_LAPTOP
        ADDITIONAL = ADDITIONAL_LAPTOP

    return (CORE, ADDITIONAL)


class ResultsRecordImporter:

    def __init__(self, record, release, logger=sys.stdout):
        self.record = record
        self.logger = logger
        self.SKIPPABLE_TESTS = get_skippable_tests(release)

    def _get_device(self, d):
        device, created = SystemDevice.objects \
                        .get_or_create(identifier=d['device']['identifier'])
        if created:
            device.name = d['device']['name']
            device.vendor_name = d['device']['vendor_name']
            if device.vendor_name is None or device.vendor_name == '':
                device.vendor_name = 'Unknown'
            if device.name is None or device.name == '':
                device.name = 'Unknown'
            device.category = d['category']
            device.save()
        return device

    def _get_profiling_devices(self):
        r = self.record.get_record()
        devices = []
        if not r:
            return devices
        for d in r['devices']:
            if d['category'] in PROFILING_CATEGORIES:
                device = self._get_device({'category': d['category'],
                    'device': d})
                if device not in devices:
                    devices.append(device)
        return SystemDevice.objects.filter(pk__in=[d.id for d in devices])

    def _import_system(self):
        r = self.record.get_record()
        profiling_devices = self._get_profiling_devices()
        if not profiling_devices:
            return False
        release = Distro.objects.get_distro(r['release'])
        # config that matches and has an exact match on these devices?
        sig = generate_device_signature(profiling_devices)
        config, created = SystemConfig.objects.get_or_create(
                                make=r['make'],
                                model=r['model'],
                                release=release,
                                device_profile_signature=sig)
        if created:
            config.form_factor = r['form_factor']
            for d in profiling_devices:
                config.device_profile.add(d)
            config.save()

        return config

    def _import_unit(self, system):
        r = self.record.get_record()
        if not r['test_results']:
            return None

        CORE, ADDITIONAL = get_core_and_additional_categories(system)
        all_categories = set(CORE + ADDITIONAL)
        all_tested_categories = set([x['category'] for x in r['test_results']])
        if len(all_categories.intersection(all_tested_categories)) == 0:
            RejectedSubmission.objects.filter(
                test_run_source=r['test_run_source']).delete()
            RejectedSubmission.objects.create(
                test_run_source=r['test_run_source'],
                submitted_by=r['submitted_by'],
                submission_date=r['submission_date'],
                reason='No applicable tests')
            return None

        unit, created = SystemUnit.objects.get_or_create(
                        config=system, identifier=r['unit_identifier'])
        unit.test_run_source = r.get('test_run_source', '')
        unit.submitted_by = r['submitted_by']
        unit.submission_date = r['submission_date']
        unit.refreshed = True
        unit.save()
        results = []

        for d in r['test_results']:
            if d['category'] not in CORE and d['category'] not in ADDITIONAL:
                # we don't care about this test
                continue
            if 'device' in d:
                device = self._get_device(d)
                result = SystemUnitTestResult(
                            device=device,
                            unit=unit,
                            category=d['category'])
            else:
                result = SystemUnitTestResult(
                            unit=unit,
                            category=d['category'])

            ignore_test = False
            # now work out the status of this result
            # we keep these details because they are the translated
            # details with UF categories, and this is used to work
            # out the scoring.
            result.skipped = '\n'.join(d['skipped']).strip()
            result.failed = '\n'.join(d['failed']).strip()
            result.passed = '\n'.join(d['passed']).strip()

            # if there are no passes or fails, it's been skipped
            if not d['passed'] and not d['failed']:
                result.status = SKIPPED

            # if we've skipped a non skippable core test, we fail
            if d['category'] in CORE:
                for t in d['skipped']:
                    if not t in self.SKIPPABLE_TESTS:
                        result.status = FAILED
            # record the skipped tests
            SkippedTest.objects.filter(test_run_source=unit.test_run_source) \
                               .delete()
            for t in d['skipped']:
                st, c = SkippedTest.objects.get_or_create(
                             test_run_source=unit.test_run_source, test_name=t)
                if c:
                    st.submission_date = r['submission_date']
                    st.save()
            for t in d['failed']:
                test_name = t.split('/')[-1]
                # detect tests are there to see if we have hardware
                # if they fail, we have no hardware to test, so ignore
                if test_name.startswith('detect'):
                    ignore_test = True
                else:
                    result.status = FAILED

            # we don't care if you skip additional category tests
            if result.status == SKIPPED and result.category in ADDITIONAL:
                ignore_test = True

            # core categories that only skipped tests
            # might be ok, we check if all the tests are skippable
            if result.category in CORE and d['skipped']:
                all_tests_skippable = True
                non_skippable_skipped = []
                RejectedSubmission.objects\
                    .filter(test_run_source=r['test_run_source']).delete()
                for t in d['skipped']:
                    all_tests_skippable = all_tests_skippable & \
                        (t in self.SKIPPABLE_TESTS)
                    if t not in self.SKIPPABLE_TESTS:
                        non_skippable_skipped.append(t)
                if all_tests_skippable:
                    result.status = SKIPPED
                    ignore_test = False
                else:
                    reject = RejectedSubmission(
                        test_run_source=r['test_run_source'],
                        submitted_by=r['submitted_by'],
                        submission_date=r['submission_date'],
                        reason='Skipped:\n' + '\n'.join(non_skippable_skipped)
                    )
                    reject.save()
                    # user has skipped a core test category.
                    # Reject the submision
                    if created:
                        unit.delete()
                    # check if the system has no units, if so
                    # remove it because there is no data for it
                    return None

            if not ignore_test:
                # if we've get here without being marked as failed or skipped
                # and the failed list is empty, we must have passed.
                if not d['failed'] and result.status == UNKNOWN:
                    result.status = PASSED
                results.append(result)
        SystemUnitTestResult.objects.filter(unit=unit).delete()
        for res in results:
            res.save()
        for d in r['devices']:
            tmp = {'device': d, 'category': d['category']}
            device = self._get_device(tmp)
            system.all_devices.add(device)
        return unit

    # TODO: move this into a util module
    def _save_star_rating(self, unit, system):
        from decimal import Decimal
        core_count = 0
        core_passed = 0
        additional_count = 0
        additional_passed = 0
        penalties = Decimal('0.00')
        results = SystemUnitTestResult.objects.filter(unit=unit)

        CORE, ADDITIONAL = get_core_and_additional_categories(system)

        for r in results:
            if r.category in CORE:
                if r.status == SKIPPED:
                    penalties = penalties + Decimal('0.3')

                if r.status in (PASSED, FAILED):
                    core_count = core_count + 1

                if r.status == PASSED:
                    core_passed = core_passed + 1
                    skipped = r.skipped.split('\n')
                    for s in skipped:
                        if s in self.SKIPPABLE_TESTS:
                            penalties = penalties + Decimal('0.3')
                            break

            elif r.category in ADDITIONAL:
                additional_count = additional_count + 1
                if r.status == PASSED:
                    additional_passed = additional_passed + 1

        x = int(core_count == core_passed)
        y = additional_passed
        m = additional_count
        if m == 0:
            m = 1
            y = 1
        rating = Decimal('%s' % ((2.0 * x * ((y / float(m)) + 1.0)) + 1.0)) \
                            - penalties
        if rating < Decimal('1'):
            rating = Decimal('1')
        unit.rating = '%s' % rating
        unit.save()
        save_config_star_rating(system)

    def import_record(self):
        if not self.record:
            return False
        system = self._import_system()
        if system:
            unit = self._import_unit(system)
            if unit is None:
                if SystemUnit.objects.filter(config=system) \
                    .count() == 0:
                    system.delete()
                return None
            self._save_star_rating(unit, system)
        return system


MAKES_TO_TAKE_VERSION = (
    'lenovo'
)


class ResultsRecordTranslator:
    """
    Takes a test run from the results tracker api and transforms it
    into a record for importing by our importer
    """

    def __init__(self, test_run=None, release=None, logger=sys.stdout):
        self.test_run = test_run
        self.release = release
        self.logger = logger
        self.CATEGORY_MAPPINGS = get_category_mappings(release)
        self.INDIVIDUAL_TEST_MAPPINGS = get_test_mappings(release)
        self.transformed_record = False
        self._transform_record()

    def _get_form_factor(self, test_run):
        for t in test_run.searchTestResults():
            if t.name == 'miscellanea/is_laptop':
                if t.status == 'Pass':
                    return 'laptop'
        return 'desktop'

    def _transform_unit(self, test_run):
        unit = test_run.system_unit
        self.logger.write('%s\n' % unit)
        if unit is None or \
            not unit.system.make or \
            not unit.system.model:
            return False
        u = {}
        u['make'] = unit.system.make
        u['model'] = unit.system.model
        if u['make'].lower() in MAKES_TO_TAKE_VERSION:
            u['model'] = unit.system.version
        if not u['model']:
            return False
        u['unit_identifier'] = unit.self_link
        u['form_factor'] = self._get_form_factor(test_run)
        u['release'] = self.release.release
        u['test_run_source'] = test_run.self_link
        u['submitted_by'] = 'Unknown'
        if test_run.person:
            u['submitted_by'] = test_run.person.name
        u['submission_date'] = test_run.date_created
        self.logger.write('%s\n' % test_run.self_link)
        u['devices'] = []
        for d in unit.getDeviceStates():
            if d.category_name is not None and \
                d.product_name is not None:
                u['devices'].append(self._get_device_from_state(d))
        if not u['devices']:
            return False
        processors = list(unit.getProcessorStates())
        if processors:
            u['devices'].append(
                self._get_processor_device_from_state(processors[0]))
        return u

    def _generate_device_id(self, device):
        data = [device.bus_name, device.product_id]
        if device.vendor_id is not None:
            data.append(device.vendor_id)
        if device.subproduct_id is not None:
            data.append(device.subproduct_id)
        if device.subvendor_id is not None:
            data.append(device.subvendor_id)
        return ':'.join(map(str, data))

    def _get_processor_device_from_state(self, p):
        vendor = p.make.replace('Genuine', '').replace('Authentic', '')
        model = p.model
        x = re.compile('[^0-9]')
        return {'identifier': '%s:%s' % (vendor, x.sub('', model)),
                'name': model,
                'vendor_name': vendor,
                'category': self._get_category('CPU')}

    def _get_device_from_state(self, d):
        p_name = d.product_name
        v_name = d.vendor_name
        category = d.category_name.lower()
        return {'identifier': self._generate_device_id(d),
                    'name': p_name,
                    'vendor_name': v_name,
                    'category': self._get_category(category)}

    def _get_devices(self, category, record):
        devices = []
        category = self._get_category(category)
        for d in record['devices']:
            if d['category'] is None:
                continue
            if d['category'].lower() == category.lower():
                devices.append(d)
        return devices

    def _get_category(self, category_name):
        if category_name is None:
            return 'unknown'
        category_name = category_name.strip()
        category_name = category_name.lower()
        return self.CATEGORY_MAPPINGS.get(category_name, category_name)

    def _transform_test_result(self, result, record):
        test_name = result.name
        tmp = test_name.split('/')
        if len(tmp) < 2:
            return []
        category = self._get_category(tmp[0])
        status = result.status
        if status != 'Pass' and status != 'Fail' and status != 'Untested':
            return False
        if test_name in self.INDIVIDUAL_TEST_MAPPINGS:
            test_name = self.INDIVIDUAL_TEST_MAPPINGS[test_name]
            category = test_name.split('/')[0]
        else:
            test_name = category + '/' + tmp[1]
        rows = []
        # also return the name of the test with the UF category
        r = {'category': category,
            'status': status,
            'test_name': test_name}
        devices = self._get_devices(category, record)
        if devices:
            for d in devices:
                row = r.copy()
                row['device'] = d
                rows.append(row)
        else:
            rows.append(r)
        return rows

    def _transform_record(self):
        if self.test_run is None:
            return
        record = self._transform_unit(self.test_run)
        if not record:
            return
        record['test_results'] = []
        for test in self.test_run.searchTestResults():
            test_results = self._transform_test_result(test, record)
            if test_results:
                record['test_results'].extend(test_results)
        # now reduce the test results on category/device so we only
        # have one list of tests for each category/device combination
        # TODO: do not include skipped tests, add them to the skipped list
        reduce = {}
        for tr in record['test_results']:
            k = tr['category']
            if not k in reduce:
                reduce[k] = {'category': k,
                    'skipped': [], 'failed': [],
                    'passed': []}
            if tr['status'] == 'Fail':
                reduce[k]['failed'].append(tr['test_name'])
            elif tr['status'] == 'Untested':
                reduce[k]['skipped'].append(tr['test_name'])
            elif tr['status'] == 'Pass':
                reduce[k]['passed'].append(tr['test_name'])
        record['test_results'] = []
        for tr in reduce.values():
            record['test_results'].append(tr)
            self.logger.write('%s\n' % tr)
        self.transformed_record = record

    def get_record(self):
        return self.transformed_record

"""
# import
from releases.models import Distro
from results.api import ResultsImporter
d = Distro.objects.get_distro('11.10')
i = ResultsImporter(d)
i.import_results()

# useful for debugging
from launchpadlib.launchpad import Launchpad
from django.conf import settings
api = Launchpad.login_anonymously('Launchpad Results', settings.LP_ROOT)
r_distro = api.distributions['ubuntu']
distro_series = r_distro.getSeries(name='oneiric')
runs = distro_series.searchTestRuns()
"""


class ResultsImporter(object):
    """
    Imports all test runs from the results tracker for a release of ubuntu,
    optionally tests run from a certain time onwards (once the api supports it)
    """

    def __init__(self, release, age=2, logger=sys.stdout, lock='cron'):
        self.release = release
        self.age = age
        self.logger = logger
        self.lock = lock
        self.yield_log = False

    def _set_up_api(self):
        """
        Make a connection to the results tracker. Puts the api object
        in self.api
        """
        # Make launchpadlib happy
        try:
            import lazr.restful
        except ImportError:
            pass

        if hasattr(settings, 'LAUNCHPAD_CACHE_DIR'):
            launchpad_cache_dir = settings.LAUNCHPAD_CACHE_DIR
        else:
            launchpad_cache_dir = None

        from launchpadlib.launchpad import Launchpad
        self.api = Launchpad.login_anonymously(
                    'Launchpad Results', settings.LP_ROOT,
                    launchpad_cache_dir)

    def _get_test_runs(self, distro_series, test_run_sequence=None):
        import datetime
        r_distro = self.api.distributions['ubuntu']
        distro_series = r_distro.getSeries(name=distro_series.codename)
        self.logger.write('%s\n' % (datetime.datetime.now() - datetime.timedelta(hours=self.age)))
        if test_run_sequence is not None:
            return distro_series.searchTestRuns(sequence=int(test_run_sequence))
        else:
            return distro_series.searchTestRuns(
                created_after=datetime.datetime.now() - \
                    datetime.timedelta(hours=self.age))

    def _flush_log(self):
        pass

    def _do_import_results(self, schedule, complete_refresh=False, test_run_sequence=None):
        lock = get_lock(self.lock)
        self._set_up_api()
        if lock:
            try:
                SystemUnit.objects.filter(config__release=self.release).update(refreshed=False)
                runs = self._get_test_runs(self.release, test_run_sequence)
                if runs is not None and list(runs):
                    for run in runs:
                        adapter = ResultsRecordTranslator(run, self.release, self.logger)
                        if not adapter.get_record():
                            continue
                        importer = ResultsRecordImporter(adapter, self.release, self.logger)
                        if importer.import_record():
                            yield self._flush_log()
            except Exception, e:
                print e
            finally:
                if complete_refresh:
                    # delete all systems that either weren't refreshed or have a rating of 0
                    to_delete = SystemUnit.objects.filter(
                        Q(refreshed=False, config__release=self.release) |
                        Q(config__release=self.release, rating=0))
                    # recalculate the affected system configs
                    systems = SystemConfig.objects.filter(systemunit__in=list(to_delete))
                    to_delete.delete()
                    # remove any configs that now have no system units
                    SystemConfig.objects.filter(
                        release=self.release, systemunit__isnull=True).delete()
                    # recalculate the survivors
                    for s in systems:
                        save_config_star_rating(s)
                schedule.save()
                lock.delete()

    def import_results(self, schedule, complete_refresh=False, test_run_sequence=None):
        for r in self._do_import_results(schedule, complete_refresh, test_run_sequence):
            pass
