from django.test import TestCase
from decimal import Decimal
from results.api import (
    ResultsImporter,
    ResultsRecordTranslator,
    ResultsRecordImporter,
)
from results.models import ImportSchedule, ImportLock
from results.management.commands.import_results import Command as ImportResultsCommand
from releases.models import Distro
from system_configs.models import (
    SystemDevice,
    SystemConfig,
    SystemUnitTestResult,
    SystemUnit,
    RejectedSubmission,
    SkippedTest,
    PASSED,
    SKIPPED,
    FAILED,
)

import datetime
import mocker


class ResultsRecordImporterTest(TestCase):

    def setUp(self):
        self.test_record = {'make': 'Dell',
                           'model': 'Vostro 3300',
                           'release': '11.10',
                           'form_factor': 'desktop',
                           'unit_identifier': 'TK421',
                           'test_run_source': 'importer-test-data',
                           'submitted_by': 'mike',
                           'submission_date': datetime.datetime.now(),
                           'devices': [
                               {'name': 'Super Ultra Graphics',
                                'identifier': 'DEV1',
                                'vendor_name': 'Vendor',
                                'category': 'video'},
                               {'name': 'Totally wireless wireless',
                                'identifier': 'DEV2',
                                'vendor_name': 'Vendor',
                                'category': 'wireless'},
                               {'name': 'ANtel Megahurtz Whizzbang cpu',
                                'identifier': 'DEV3',
                                'vendor_name': 'Vendor',
                                'category': 'cpu'},
                               {'name': 'Some misc. device',
                                'identifier': 'DEV4',
                                'vendor_name': 'Vendor',
                                'category': 'mediacards'},
                               {'name': 'A device that did not have tests',
                                'identifier': 'ZDEV1',
                                'vendor_name': 'Vendor',
                                'category': 'other'},
                           ],
                           'test_results': [
                               {'device': {
                                   'name': 'Super Ultra Graphics',
                                   'identifier': 'DEV1',
                                   'vendor_name': 'Vendor'},
                                'category': 'video',
                                'skipped': [],
                                'failed': [],
                                'passed': ['video/test']},
                               {'device': {
                                   'name': 'Totally wireless wireless',
                                   'identifier': 'DEV2',
                                   'vendor_name': 'Vendor'},
                                'category': 'wireless',
                                'skipped': [],
                                'failed': [],
                                'passed': ['network/test']},
                               {'device': {
                                   'name': 'ANtel Megahurtz Whizzbang cpu',
                                   'identifier': 'DEV3',
                                   'vendor_name': 'Vendor'},
                                'category': 'cpu',
                                'skipped': [],
                                'failed': [],
                                'passed': ['cpu/test']},
                               {'device': {
                                   'name': 'Some misc. device',
                                   'identifier': 'DEV4',
                                   'vendor_name': 'Vendor'},
                                'category': 'mediacards',
                                'skipped': [],
                                'failed': ['mediacards/read'],
                                'passed': ['mediacards/test']},
                           ]}

    def _get_importer(self, transformed_record):
        adapter = ResultsRecordTranslator(None)
        adapter.transformed_record = transformed_record
        importer = ResultsRecordImporter(adapter, Distro.objects.get_distro(transformed_record['release']))
        return importer

    def test_import_record(self):
        record = self.test_record
        importer = self._get_importer(record)
        system = importer.import_record()

        # system config imported with the right details
        self.assertEqual(system.make, record['make'])
        self.assertEqual(system.model, record['model'])
        self.assertEqual(system.form_factor, record['form_factor'])
        self.assertEqual(system.release,
            Distro.objects.get_distro(record['release']))

        # profiling devices have been imported and associated
        devices = system.device_profile.all().order_by('name')
        self.assertEqual(devices.count(), 3)
        self.assertEqual(devices[0].name, 'ANtel Megahurtz Whizzbang cpu')
        self.assertEqual(devices[0].category, 'cpu')
        self.assertEqual(devices[0].identifier, 'DEV3')
        self.assertEqual(devices[1].name, 'Super Ultra Graphics')
        self.assertEqual(devices[1].category, 'video')
        self.assertEqual(devices[1].identifier, 'DEV1')
        self.assertEqual(devices[2].name, 'Totally wireless wireless')
        self.assertEqual(devices[2].category, 'wireless')
        self.assertEqual(devices[2].identifier, 'DEV2')

        # all devices were imported
        self.assertEqual(SystemDevice.objects.all().count(), 5)
        self.assertEqual(SystemDevice.objects.filter(
            name='Some misc. device').count(), 1)

        # a system unit was created with the correct id
        self.assertEqual(SystemUnit.objects.all().count(), 1)
        self.assertEqual(SystemUnit.objects.filter(
            identifier='TK421').count(), 1)
        self.assertEqual(SystemUnitTestResult.objects.all().count(), 4)

        # device test results were imported and associated with the unit
        results = SystemUnitTestResult.objects.filter(unit__identifier='TK421')
        results = results.order_by('device__name')
        self.assertEqual(results[0].device.name,
            'ANtel Megahurtz Whizzbang cpu')
        self.assertEqual(results[0].status, PASSED)
        self.assertEqual(results[1].device.name, 'Some misc. device')
        self.assertEqual(results[1].status, FAILED)
        self.assertEqual(results[2].device.name, 'Super Ultra Graphics')
        self.assertEqual(results[2].status, PASSED)
        self.assertEqual(results[3].device.name, 'Totally wireless wireless')
        self.assertEqual(results[3].status, PASSED)
        unit = SystemUnit.objects.filter(identifier='TK421')[0]
        self.assertEqual(unit.submitted_by, record['submitted_by'])
        self.assertEqual(unit.submission_date, record['submission_date'])
        self.assertEqual(unit.test_run_source, record['test_run_source'])

        # import a second record with a different unit id
        record['unit_identifier'] = 'R2D2'
        importer = self._get_importer(record)
        system = importer.import_record()
        self.assertEqual(SystemDevice.objects.all().count(), 5)

        # we should be using the same config because the video,
        # wireless, and cpu devices are the same
        self.assertEqual(SystemConfig.objects.all().count(), 1)
        # and both units are using the same config
        self.assertEqual(SystemUnit.objects.filter(config=system).count(), 2)
        # all device results are recorded
        self.assertEqual(SystemUnitTestResult.objects.all().count(), 8)

    def test_same_make_model_different_profiling_devices_creates_new_config(self):
        record = self.test_record
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 1)
        # change one of the profiling devices
        new_device = {'name': 'NVTi Ultra Hyper Real 256', 'identifier': 'EDEV5',
            'vendor_name': 'Vendor', 'category': 'video'}
        record['test_results'][0]['device'] = new_device
        record['devices'].append(new_device)
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 2)

        # remove one of the profiling devices
        del(record['test_results'][1])
        del(record['devices'][1])
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 3)

    def test_test_results_categories(self):
        record = self.test_record
        record['test_results'].append(
            {'category': 'suspend/resume',
            'passed': ['suspend/stuff'],
            'failed': ['suspend/other'],
            'skipped': []}
        )
        importer = self._get_importer(record)
        importer.import_record()
        results = SystemUnitTestResult.objects.all().order_by('category')
        self.assertEqual(results[0].category, 'cpu')
        self.assertEqual(results[1].category, 'mediacards')
        self.assertEqual(results[2].category, 'suspend/resume')
        self.assertEqual(results[3].category, 'video')
        self.assertEqual(results[4].category, 'wireless')

    def test_import_tests_without_device(self):
        record = self.test_record
        record['test_results'].append(
            {'category': 'suspend/resume',
            'skipped': [],
            'failed': ['suspend/blah'],
            'passed': ['suspend/external_thing']}
        )
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 1)
        self.assertEqual(SystemUnit.objects.all().count(), 1)
        self.assertEqual(SystemDevice.objects.all().count(), 5)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=PASSED).count(), 3)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=FAILED).count(), 2)

    def test_importing_unit_again_updates_devices(self):
        record = self.test_record
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 1)
        self.assertEqual(SystemUnit.objects.all().count(), 1)
        self.assertEqual(SystemDevice.objects.all().count(), 5)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=PASSED).count(), 3)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=FAILED).count(), 1)

        # change one of the results to True
        record['test_results'][3]['failed'] = []
        record['test_results'][3]['passed'] = ['category/test']
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 1)
        self.assertEqual(SystemUnit.objects.all().count(), 1)
        self.assertEqual(SystemDevice.objects.all().count(), 5)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=PASSED).count(), 4)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=FAILED).count(), 0)

        # append a new result
        record['test_results'].append(
            {'device': {'name': 'NVTi Ultra Hyper Real 768', 'identifier': 'DEV6', 'vendor_name': 'Vendor'},
            'category': 'firewire',
            'skipped': [],
            'failed': [],
            'passed': ['firewire/external_thing']})
        importer = self._get_importer(record)
        importer.import_record()
        self.assertEqual(SystemConfig.objects.all().count(), 1)
        self.assertEqual(SystemUnit.objects.all().count(), 1)
        self.assertEqual(SystemDevice.objects.all().count(), 6)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=PASSED).count(), 5)
        self.assertEqual(SystemUnitTestResult.objects.filter(status=FAILED).count(), 0)

    def _get_test_results(self, core, additional, core_failing=0, additional_failing=0):
        results = []
        for c in core:
            results.append({'category': c, 'passed': ['%s/test' % c], 'skipped': [], 'failed': []})
        for i in range(0, core_failing):
            results[i]['failed'].append('%s/fail' % c)
        for c in additional:
            results.append({'category': c, 'passed': ['%s/test' % c], 'skipped': [], 'failed': []})
        for i in range(len(core), len(core) + additional_failing):
            results[i]['failed'].append('%s/fail' % c)
        return results

    def test_refreshed_flag_is_set_when_reimporting(self):
        # new units are marked as refreshed
        record = self.test_record.copy()
        importer = self._get_importer(record)
        system = importer.import_record()
        units = SystemUnit.objects.filter(config=system)
        self.assertEqual(units.count(), 1)
        self.assertEqual(units[0].refreshed, True)

        # units not reimported are not marked as refreshed
        SystemUnit.objects.all().update(refreshed=False)
        record['unit_identifier'] = 'SecondUnit'
        record['test_run_source'] = 'another_test_run'
        importer = self._get_importer(record)
        system = importer.import_record()
        units = SystemUnit.objects.filter(config=system)
        self.assertEqual(units.count(), 2)
        original_unit = SystemUnit.objects.get(identifier=self.test_record['unit_identifier'])
        self.assertEqual(original_unit.refreshed, False)
        new_unit = SystemUnit.objects.get(identifier=record['unit_identifier'])
        self.assertEqual(new_unit.refreshed, True)

        # units reimported are marked as refreshed
        record = self.test_record.copy()
        importer = self._get_importer(record)
        importer.import_record()
        original_unit = SystemUnit.objects.get(identifier=self.test_record['unit_identifier'])
        self.assertEqual(original_unit.refreshed, True)

    def test_not_imported_if_no_tests(self):
        record = self.test_record
        record['test_results'] = []
        importer = self._get_importer(record)
        system = importer.import_record()
        self.assertEqual(system, None)
        self.assertEqual(SystemConfig.objects.filter(model='Vostro 3300').count(), 0)

    def test_star_rating_desktop(self):
        from results.api import CORE_DESKTOP
        all_core_half_additional_desktop = self.test_record
        all_core_half_additional_desktop['test_results'] = self._get_test_results(
            CORE_DESKTOP, ['bluetooth', 'firewire'], 0, 1)
        importer = self._get_importer(all_core_half_additional_desktop)
        system = importer.import_record()
        system = SystemConfig.objects.get(pk=system.id)
        self.assertEqual(system.num_ratings, 1)
        unit = SystemUnit.objects.filter(config=system)[0]
        self.assertEqual(unit.rating, Decimal(4))
        self.assertEqual(system.overall_rating, Decimal(4))

        all_but_one_core_all_additional_desktop = self.test_record
        all_but_one_core_all_additional_desktop['unit_identifier'] = 'E5A908B7'
        all_but_one_core_all_additional_desktop['test_results'] = self._get_test_results(
            CORE_DESKTOP, ['bluetooth', 'firewire'], 1, 0)
        importer = self._get_importer(all_but_one_core_all_additional_desktop)
        system = importer.import_record()
        system = SystemConfig.objects.get(pk=system.id)
        self.assertEqual(system.overall_rating, Decimal('2.5'))
        self.assertEqual(system.num_ratings, 2)
        unit = SystemUnit.objects.get(config=system, identifier='E5A908B7')
        self.assertEqual(unit.rating, Decimal(1))

        all_core_all_additional_desktop = self.test_record
        all_core_all_additional_desktop['unit_identifier'] = 'JAMJAMJAM'
        all_core_all_additional_desktop['test_results'] = self._get_test_results(
            CORE_DESKTOP, ['bluetooth', 'firewire'])
        importer = self._get_importer(all_core_all_additional_desktop)
        system = importer.import_record()
        system = SystemConfig.objects.get(pk=system.id)
        self.assertEqual(system.overall_rating, Decimal('3.33'))
        self.assertEqual(system.num_ratings, 3)
        unit = SystemUnit.objects.get(config=system, identifier='JAMJAMJAM')
        self.assertEqual(unit.rating, 5)

    def test_star_rating_laptop(self):
        from results.api import CORE_LAPTOP
        self.test_record['form_factor'] = 'laptop'
        all_core_half_additional_laptop = self.test_record
        all_core_half_additional_laptop['test_results'] = self._get_test_results(
            CORE_LAPTOP, ['bluetooth', 'firewire'], 0, 1)
        importer = self._get_importer(all_core_half_additional_laptop)
        system = importer.import_record()
        system = SystemConfig.objects.get(pk=system.id)
        self.assertEqual(system.num_ratings, 1)
        unit = SystemUnit.objects.filter(config=system)[0]
        self.assertEqual(unit.rating, Decimal(4))
        self.assertEqual(system.overall_rating, Decimal(4))

        all_but_one_core_all_additional_laptop = self.test_record
        all_but_one_core_all_additional_laptop['unit_identifier'] = 'E5A908B7'
        all_but_one_core_all_additional_laptop['test_results'] = self._get_test_results(
            CORE_LAPTOP, ['bluetooth', 'firewire'], 1, 0)
        importer = self._get_importer(all_but_one_core_all_additional_laptop)
        system = importer.import_record()
        system = SystemConfig.objects.get(pk=system.id)
        self.assertEqual(system.overall_rating, Decimal('2.5'))
        self.assertEqual(system.num_ratings, 2)
        unit = SystemUnit.objects.get(config=system, identifier='E5A908B7')
        self.assertEqual(unit.rating, Decimal(1))

        all_core_all_additional_laptop = self.test_record
        all_core_all_additional_laptop['unit_identifier'] = 'JAMJAMJAM'
        all_core_all_additional_laptop['test_results'] = self._get_test_results(
            CORE_LAPTOP, ['bluetooth', 'firewire'])
        importer = self._get_importer(all_core_all_additional_laptop)
        system = importer.import_record()
        system = SystemConfig.objects.get(pk=system.id)
        self.assertEqual(system.overall_rating, Decimal('3.33'))
        self.assertEqual(system.num_ratings, 3)
        unit = SystemUnit.objects.get(config=system, identifier='JAMJAMJAM')
        self.assertEqual(unit.rating, 5)

    def test_all_devices_recorded_against_config(self):
        importer = self._get_importer(self.test_record)
        system = importer.import_record()
        system_devices = system.all_devices.all()
        devices = SystemDevice.objects.all().order_by('identifier')
        self.assertEqual(devices.count(), 5)
        self.assertEqual(devices[0].name, 'Super Ultra Graphics')
        self.assertEqual(devices[0].category, 'video')
        self.assertEqual(devices[1].name, 'Totally wireless wireless')
        self.assertEqual(devices[1].category, 'wireless')
        self.assertEqual(devices[2].name, 'ANtel Megahurtz Whizzbang cpu')
        self.assertEqual(devices[2].category, 'cpu')
        self.assertEqual(devices[3].name, 'Some misc. device')
        self.assertEqual(devices[3].category, 'mediacards')
        self.assertEqual(devices[4].name, 'A device that did not have tests')
        self.assertEqual(devices[4].category, 'other')
        for d in SystemDevice.objects.all():
            self.assertTrue(d in system_devices)

    def test_skippable_core_tests_reduce_rating(self):
        record = self.test_record
        record['test_results'].append(
                                {'device': {
                                   'name': 'Jammin Audio Card',
                                   'identifier': 'AUD1',
                                   'vendor_name': 'Cre8iv Lahbs'},
                                'category': 'audio',
                                'skipped': ['audio/alsa_record_playback_external',
                                    'audio/alsa_record_playback_usb'],
                                'failed': [],
                                'passed': ['audio/external_sound']})
        importer = self._get_importer(self.test_record)
        system = importer.import_record()
        unit = SystemUnit.objects.get(config=system)
        self.assertEqual(unit.rating, Decimal('3.70'))
        # check the skipped tests are recorded
        skipped = SkippedTest.objects.filter(test_run_source=record['test_run_source'],
                                             test_name='audio/alsa_record_playback_external')
        self.assertEqual(skipped.count(), 1)
        skipped = SkippedTest.objects.filter(test_run_source=record['test_run_source'],
                                             test_name='audio/alsa_record_playback_usb')
        self.assertEqual(skipped.count(), 1)

    def test_skipping_non_skippable_core_test_rejects_submission(self):
        record = self.test_record
        record['test_results'].append(
                                {'device': {
                                   'name': 'Jammin Audio Card',
                                   'identifier': 'AUD1',
                                   'vendor_name': 'Cre8iv Lahbs'},
                                'category': 'audio',
                                'skipped': ['audio/playback'],
                                'passed': ['audio/detect', 'audio/recording'],
                                'failed': []})
        importer = self._get_importer(self.test_record)
        system = importer.import_record()
        # no unit should have been created
        self.assertEqual(SystemUnit.objects.all().count(), 0)
        # the system should not have been created either
        self.assertEqual(system, None)
        # rejection is recorded
        rejection = RejectedSubmission.objects.get(test_run_source=record['test_run_source'])
        self.assertEqual(rejection.reason, 'Skipped:\naudio/playback')
        self.assertEqual(rejection.submission_date, record['submission_date'])
        self.assertEqual(rejection.submitted_by, record['submitted_by'])

    def test_skipping_all_tests_in_core_category_rejects_submission(self):
        record = self.test_record
        record['test_results'].append(
                                {'device': {
                                   'name': 'Jammin Audio Card',
                                   'identifier': 'AUD1',
                                   'vendor_name': 'Cre8iv Lahbs'},
                                'category': 'audio',
                                'skipped': ['audio/playback', 'audio/detect', 'audio/recording'],
                                'passed': [],
                                'failed': []})
        importer = self._get_importer(self.test_record)
        system = importer.import_record()
        # no unit should have been created
        self.assertEqual(SystemUnit.objects.all().count(), 0)
        # the system should not have been created either
        self.assertEqual(system, None)
        # rejection is recorded
        rejection = RejectedSubmission.objects.get(test_run_source=record['test_run_source'])
        self.assertEqual(rejection.reason, 'Skipped:\naudio/playback\naudio/detect\naudio/recording')
        self.assertEqual(rejection.submission_date, record['submission_date'])
        self.assertEqual(rejection.submitted_by, record['submitted_by'])

    def test_detect_failed_category_ignores_category(self):
        record = self.test_record
        record['test_results'].append(
                                {'device': {
                                   'name': 'Jammin Audio Card',
                                   'identifier': 'AUD1',
                                   'vendor_name': 'Cre8iv Lahbs'},
                                'category': 'audio',
                                'failed': ['audio/detect'],
                                'skipped': [],
                                'passed': []})
        importer = self._get_importer(self.test_record)
        system = importer.import_record()
        unit = SystemUnit.objects.get(config=system)
        audio_result = SystemUnitTestResult.objects.filter(unit=unit, category='audio')
        self.assertEqual(audio_result.count(), 0)

    def test_no_passed_tests_and_no_failed_tests_get_ignored_in_additional_category(self):
        record = self.test_record
        record['test_results'].append(
                                {'device': {
                                   'name': 'Firewire device',
                                   'identifier': 'FWD1',
                                   'vendor_name': 'Inferno Wire'},
                                'category': 'firewire',
                                'failed': [],
                                'passed': [],
                                'skipped': ['firewire/tests']})
        importer = self._get_importer(self.test_record)
        system = importer.import_record()
        unit = SystemUnit.objects.get(config=system)
        audio_result = SystemUnitTestResult.objects.filter(unit=unit,
            category='firewire')
        self.assertEqual(audio_result.count(), 0)

    def test_core_category_with_all_skippable_tests(self):
        record = self.test_record
        record['test_results'].append(
                                {'category': 'vga external video',
                                'failed': [],
                                'passed': [],
                                'skipped': ['vga external video/hdmi', 'vga external video/vga']})
        importer = self._get_importer(record)
        system = importer.import_record()
        unit = SystemUnit.objects.get(config=system)
        vga_result = SystemUnitTestResult.objects.get(unit=unit,
            category='vga external video')
        self.assertEqual(vga_result.status, SKIPPED)
        # penalty applied for the skipped category
        self.assertEqual(unit.rating, Decimal('3.70'))

    def test_some_results_but_no_core_or_additional_categories(self):
        """
        If there are test results for categories that are not core or
        additional, and there are not results for core or additional categories,
        the submission is rejected as having no applicable tests.
        """
        record = self.test_record
        record['test_results'] = [{'category': 'unity',
                                   'skipped': [],
                                   'failed': [],
                                   'passed': ['unity/test']},
                                  {'category': 'wm',
                                   'skipped': [],
                                   'failed': [],
                                   'passed': ['wm/test']}]
        importer = self._get_importer(record)
        system = importer.import_record()
        # no unit should have been created
        self.assertEqual(SystemUnit.objects.all().count(), 0)
        # the system should not have been created either
        self.assertEqual(system, None)
        self.assertEqual(SystemConfig.objects.all().count(), 0)
        # rejection is recorded
        rejection = RejectedSubmission.objects.get(
            test_run_source=record['test_run_source'])
        self.assertEqual(rejection.reason, 'No applicable tests')


class ImportResultsCommandTest(mocker.MockerTestCase, TestCase):

    def setUp(self):
        self.now = datetime.datetime(2012, 4, 19, 0, 0, 0, 0)
        self.mock_datetime = self.mocker.replace(datetime)
        Distro.objects.all().delete()
        self.release = Distro.objects.create(codename='lucid', release='10.04')

    def test_schedule_creation(self):
        """
        On first import for a release, an import schedule is created
        with the import_since as the current time. A complete refresh is performed.
        """
        self.mock_importer = self.mocker.patch(ResultsImporter)
        # setting up default dates in models and the importer
        self.mock_datetime.datetime.now()
        self.mocker.result(self.now)
        self.mock_datetime.datetime.now()
        self.mocker.result(self.now)
        # check the importer gets complete_refresh, persist the schedule
        self.mock_importer.import_results(
            mocker.ANY, complete_refresh=True, test_run_sequence=None)
        self.mocker.call(
            lambda schedule, complete_refresh, test_run_sequence: schedule.save())
        self.mocker.replay()

        cmd = ImportResultsCommand()
        cmd.handle_noargs()
        schedules = ImportSchedule.objects.all()
        self.assertEqual(schedules.count(), 1)
        self.assertEqual(schedules[0].release, self.release)
        self.assertEqual(schedules[0].import_since, self.now)
        self.assertEqual(schedules[0].full_refresh, False,
            'Schedule has full_refresh as True, meaning next time'
            ' will be a full refresh also.')

    def test_schedule_records_time_of_last_import(self):
        """
        Running the import command records the time that the import
        was run in the release's import schedule (so that we know
        when to import records from next time)
        """
        self.mock_importer = self.mocker.patch(ResultsImporter)
        schedule = ImportSchedule.objects.create(release=self.release)
        schedule.import_since = datetime.datetime(2001, 1, 1, 0, 0)
        schedule.save()
        # setting up default dates in models and the importer
        self.mock_datetime.datetime.now()
        self.mocker.result(self.now)
        self.mock_importer.import_results(
            schedule, complete_refresh=False, test_run_sequence=None)
        # persist the schedule when it's passed to import_results
        self.mocker.call(
            lambda schedule, complete_refresh, test_run_sequence: schedule.save())
        self.mocker.replay()

        cmd = ImportResultsCommand()
        cmd.handle_noargs()

        schedule = ImportSchedule.objects.get(release=self.release)
        self.assertEqual(schedule.import_since, self.now)

    def test_schedule_with_full_refresh_set_will_set_time_to_uf_epoch(self):
        """
        The UF program began on a certain date, so when refreshing all data
        that date will be passed to the importer as the beginning date
        to get records from.
        """
        schedule = ImportSchedule.objects.create(release=self.release, full_refresh=True)
        schedule.import_since = datetime.datetime(2001, 1, 1, 0, 0)
        schedule.save()
        epoch = datetime.datetime(2011, 9, 1)
        delta = self.now - epoch
        expected_age = int(delta.days * 24 + delta.seconds / 60.0 / 60.0 + 1)
        # setting up default dates in models and the importer
        self.mock_importer = self.mocker.replace(ResultsImporter)
        self.mock_datetime.datetime.now()
        self.mocker.result(self.now)

        import_results_mock = self.mocker.mock()
        import_results_mock.import_results(
            schedule, complete_refresh=True, test_run_sequence=None)
        self.mocker.call(
            lambda schedule, complete_refresh, test_run_sequence: schedule.save())

        # persist the schedule when it's passed to import_results
        self.mock_importer(self.release, expected_age)
        self.mocker.result(import_results_mock)
        self.mocker.replay()

        cmd = ImportResultsCommand()
        cmd.handle_noargs()

        schedule = ImportSchedule.objects.get(pk=schedule.id)
        self.assertEqual(schedule.import_since, self.now,
            'Date to import since was not set to current datetime')
        self.assertEqual(schedule.full_refresh, False,
            'full_refresh flag was not reset to False')
