from django.db import models
from releases.models import Distro
from system_configs.managers import SubmissionManager

FORM_FACTORS = (
    ('desktop', 'desktop'),
    ('laptop', 'laptop'),
)

PASSED = 'passed'
FAILED = 'failed'
SKIPPED = 'skipped'
UNKNOWN = 'unknown'


def round_rating(rating):
    rating = float(rating)
    rating = int(rating * 10)
    if rating % 5 > 2:
        rating = rating + 5 - (rating % 5)
    else:
        rating = rating - (rating % 5)
    return '%s' % (rating / 10.0)


class BaseConverter(object):
    decimal_digits = "0123456789"

    def __init__(self, digits):
        self.digits = digits

    def from_decimal(self, i):
        return self.convert(i, self.decimal_digits, self.digits)

    def to_decimal(self, s):
        return int(self.convert(s, self.digits, self.decimal_digits))

    def convert(number, fromdigits, todigits):
        # Based on http://code.activestate.com/recipes/111286/
        if str(number)[0] == '-':
            number = str(number)[1:]
            neg = 1
        else:
            neg = 0

        # make an integer out of the number
        x = 0
        for digit in str(number):
            x = x * len(fromdigits) + fromdigits.index(digit)

        # create the result in base 'len(todigits)'
        if x == 0:
            res = todigits[0]
        else:
            res = ""
            while x > 0:
                digit = x % len(todigits)
                res = todigits[digit] + res
                x = int(x / len(todigits))
            if neg:
                res = '-' + res
        return res
    convert = staticmethod(convert)

base62 = BaseConverter(
    'ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789abcdefghijklmnopqrstuvwxyz'
)


def generate_device_signature(devices):
    sig = ''
    for d in devices:
        # type, will be v, w, or the first letter of the processor vendor
        sig = sig + d.identifier[0]
        tmp = d.identifier.split(':')
        for i in tmp[1:]:
            if i == '':
                i = '0'
            sig = sig + ':' + base62.from_decimal(i)
    return sig


class SystemDevice(models.Model):
    identifier = models.CharField(max_length=96)
    name = models.CharField(max_length=96)
    vendor_name = models.CharField(max_length=96, null=True)
    category = models.CharField(max_length=48)

    def vendor(self):
        # bug 851164
        return self.vendor_name.split(' ')[0]

    def __unicode__(self):
        return u'%s %s' % (self.vendor_name, self.name)

    class Meta:
        ordering = ('category', 'name', 'identifier', )


class SystemConfig(models.Model):
    make = models.CharField(max_length=64)
    model = models.CharField(max_length=64)
    release = models.ForeignKey(Distro)
    form_factor = models.CharField(max_length=7, choices=FORM_FACTORS,
                    blank=True, null=True)
    device_profile = models.ManyToManyField(SystemDevice,
        related_name='configprofile')
    device_profile_signature = models.TextField()
    all_devices = models.ManyToManyField(SystemDevice)
    num_ratings = models.IntegerField(default=0)
    overall_rating = models.DecimalField(
        max_digits=3, decimal_places=2, default=0)
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    class Meta:
        unique_together = [('make', 'model', 'device_profile_signature',
                            'release', 'form_factor')]

    def rounded_rating(self):
        return round_rating(self.overall_rating)

    def stars_css(self):
        css = '%s' % self.rounded_rating()
        css = css.replace('.', '-')
        return css

    def stars_breakdown(self):
        from decimal import Decimal
        total = SystemUnit.objects.filter(config=self).count()
        breakdown = []
        for x in range(1, 6):
            breakdown.insert(0,
                {'stars': x,
                 'count': SystemUnit.objects.filter(
                    config=self,
                    rating__range=(x, Decimal('%s' % (x + 0.99)))).count(),
                 'total': total})
        return breakdown

    def all_releases(self):
        return SystemConfig.objects \
            .filter(device_profile_signature=self.device_profile_signature)\
            .filter(make=self.make, model=self.model)\
                .order_by('-release__i_version')

    def __unicode__(self):
        return u'%s %s' % (self.make, self.model)

    @models.permalink
    def get_absolute_url(self):
        return ('system_configs.views.system_config',
            (), {'release': self.release.release,
                'make': self.make,
                'model': self.model.replace('#', '%23'),
                'device_sig': self.device_profile_signature})

    @models.permalink
    def all_devices_url(self):
        return ('system_configs.views.all_devices',
            (), {'release': self.release.release,
                'make': self.make,
                'model': self.model.replace('#', '%23'),
                'device_sig': self.device_profile_signature})

    @models.permalink
    def status_url(self):
        return ('system_configs.views.status',
            (), {'release': self.release.release,
                'make': self.make,
                'model': self.model.replace('#', '%23'),
                'device_sig': self.device_profile_signature})

    @models.permalink
    def feedback_url(self):
        return ('system_configs.views.feedback',
            (), {'release': self.release.release,
                'make': self.make,
                'model': self.model.replace('#', '%23'),
                'device_sig': self.device_profile_signature})


class TestRunMixin(models.Model):
    test_run_source = models.TextField()
    submitted_by = models.TextField()
    submission_date = models.DateTimeField(null=True)

    objects = SubmissionManager()

    class Meta:
        abstract = True


class SystemUnit(TestRunMixin):
    config = models.ForeignKey(SystemConfig)
    identifier = models.CharField(max_length=96)
    rating = models.DecimalField(max_digits=3, decimal_places=2, default=0)
    # used when doing a full refresh of the database
    refreshed = models.BooleanField(default=True)
    created = models.DateTimeField(auto_now_add=True)

    def rounded_rating(self):
        return round_rating(self.rating)


class RejectedSubmission(TestRunMixin):
    reason = models.TextField()


class SkippedTest(TestRunMixin):
    test_name = models.TextField()

    class Meta:
        unique_together = ('test_run_source', 'test_name')


class SystemUnitTestResult(models.Model):
    unit = models.ForeignKey(SystemUnit)
    device = models.ForeignKey(SystemDevice, null=True)
    category = models.CharField(max_length=24)
    status = models.CharField(max_length=8, default=UNKNOWN)
    passed = models.TextField()
    skipped = models.TextField()
    failed = models.TextField()
    created = models.DateTimeField(auto_now_add=True)

    class Meta:
        ordering = ('category',)
