from django.db.models import Count
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.core.urlresolvers import reverse
from results.api import (
    PROFILING_CATEGORIES,
    get_core_and_additional_categories,
)
from operator import itemgetter
from system_configs.models import (
    SystemConfig,
    SystemUnit,
    SystemUnitTestResult,
    PASSED,
    FAILED,
)


def _get_test_results(system, categories):
    # the following stats code should be in a model somewhere.
    raw_test_stats = SystemUnitTestResult.objects.filter(unit__config=system) \
                         .filter(category__in=categories) \
                         .filter(status__in=(PASSED, FAILED)) \
                         .values('category', 'status') \
                         .annotate(Count('id')).order_by('category', 'status')
    test_results = {}

    for stat in raw_test_stats:
        category = stat['category']
        if not category in test_results:
            test_results[category] = {'category': category,
                'passes': 0, 'fails': 0, 'total': 0, 'status': 'unknown'}
        if stat['status'] == PASSED:
            test_results[category]['passes'] = stat['id__count']
        elif stat['status'] == FAILED:
            test_results[category]['fails'] = stat['id__count']

    # squish the data down to one row per category,
    # with totals and percentages and statuses
    for category in test_results:
        test_results[category]['total'] = test_results[category]['passes'] \
            + test_results[category]['fails']
        test_results[category]['percentage'] = float(
            test_results[category]['passes']) \
            / test_results[category]['total'] * 100
        p = test_results[category]['percentage']
        if p >= 75:
            test_results[category]['status'] = 'working'
        if p < 75 and p > 25:
            test_results[category]['status'] = 'partial'
        if p < 25:
            test_results[category]['status'] = 'not-working'

    return test_results


def _load_profile_devices_with_results(profile_devices, test_results):
    categories = test_results.keys()
    categories.sort()
    for category in categories:
        for d in profile_devices:
            if d.category == category:
                d.test_results = test_results[category]


def _category_cmp(x, y):
    if x in PROFILING_CATEGORIES and y not in PROFILING_CATEGORIES:
        return -1
    if x not in PROFILING_CATEGORIES and y in PROFILING_CATEGORIES:
        return 1
    if x < y:
        return -1
    if y < x:
        return 1
    return 0


def system_config(request, release, make, model, device_sig):
    def results_row(category, result):
        row = {}
        row['category'] = category
        row['devices'] = system.all_devices.filter(category=category)
        row['result'] = result
        return row
    system = get_object_or_404(SystemConfig, make=make, model=model,
        device_profile_signature=device_sig, release__release=release)
    CORE, ADDITIONAL = get_core_and_additional_categories(system)
    core_test_results = _get_test_results(system, CORE)
    additional_test_results = _get_test_results(system, ADDITIONAL)
    profile_devices = system.device_profile.all()
    _load_profile_devices_with_results(profile_devices, core_test_results)

    similar_systems = SystemConfig.objects.filter(make=system.make,
                                                  model=system.model,
                                                  release=system.release) \
                                          .exclude(pk=system.id) \
                                          .order_by('-overall_rating')
    
    core_device_statuses = []
    for t in core_test_results:
        core_device_statuses.append(results_row(t, core_test_results[t]))
    additional_device_statuses = []
    for t in additional_test_results:
        additional_device_statuses.append(results_row(t, additional_test_results[t]))
    core_device_statuses = sorted(core_device_statuses,
        key=itemgetter('category'),
        cmp = _category_cmp)
    additional_device_statuses = sorted(additional_device_statuses,
        key=itemgetter('category'),
        cmp = _category_cmp)
    if request.GET.get('show_sources', False):
        test_sources = system.systemunit_set.all().values('test_run_source')
    context = RequestContext(request, locals())
    return render_to_response('system_config.html', context)


def units(request, release, make, model, device_sig):
    system = get_object_or_404(SystemConfig, make=make, model=model,
        release__release=release, device_profile_signature=device_sig)
    system_units = SystemUnit.objects.filter(config=system)
    context = RequestContext(request, locals())
    return render_to_response('system_config_units.html', context)


def unit(request, release, make, model, device_sig, unit_id):
    system = get_object_or_404(SystemConfig, make=make, model=model,
        release__release=release, device_profile_signature=device_sig)
    system_unit = SystemUnit.objects.get(config=system, pk=unit_id)
    components = SystemUnitTestResult.objects.filter(unit=system_unit)
    context = RequestContext(request, locals())
    return render_to_response('system_config_unit.html', context)


def _get_mobile_tabs(release, system):
    tabs = []
    tabs.append({'label': 'User results',
            'url': system.status_url()})
    tabs.append({'label': 'Configuration',
            'url': system.all_devices_url()})
    tabs.append({'label': 'Search',
            'url': reverse('browse.views.index')})
    return tabs


def status(request, release, make, model, device_sig):
    system = get_object_or_404(SystemConfig, make=make, model=model,
        release__release=release, device_profile_signature=device_sig)
    CORE, ADDITIONAL = get_core_and_additional_categories(system)
    tmp = _get_test_results(system, CORE + ADDITIONAL)
    test_results = []
    for category in tmp:
        test_results.append(tmp[category])
    test_results = sorted(test_results, key=itemgetter('category'))
    context = RequestContext(request,
        {'test_results': test_results, 'system': system})
    if request.mobile:
        tabs = _get_mobile_tabs(release, system)
        tabs[0]['active'] = True
        context = RequestContext(request, locals())
        return render_to_response('mobile/system_test_stats.html', context)
    return render_to_response('system_test_stats.html', context)


def feedback(request, release, make, model, device_sig):
    system = get_object_or_404(SystemConfig, make=make, model=model,
        release__release=release, device_profile_signature=device_sig)
    context = RequestContext(request, locals())
    return render_to_response('report_a_problem.html', context)


def all_devices(request, release, make, model, device_sig):
    system = get_object_or_404(SystemConfig, make=make, model=model,
        release__release=release, device_profile_signature=device_sig)
    devices = system.all_devices.all() \
                .exclude(pk__in=system.device_profile.all()) \
                .exclude(name='') \
                .order_by('vendor_name','name')
    profile_devices = system.device_profile.all()
    if request.mobile:
        tabs = _get_mobile_tabs(release, system)
        tabs[1]['active'] = True
        context = RequestContext(request, locals())
        return render_to_response('mobile/system_all_devices.html', context)
    context = RequestContext(request, locals())
    return render_to_response('system_all_devices.html', context)
